#include "nummatrix.h"
#include <vector>
#include <cmath>

template<class M, class T>
void gaussseidel (const M& A, const std::vector<T> & b, std::vector<T> & x,
    int iter)
{
    for (int i = 0; i< iter; i++)
    {
        for (typename M::ConstRowIterator rit = A.begin();
             rit != A.end(); ++rit)
        {
            x[rit.row()] = b[rit.row()];
            T diag;
            for (typename M::ConstColIterator cit = rit->begin();
                 cit != rit->end(); ++cit)
            {
                if (cit.col() != rit.row())
                    x[rit.row()] -= (*cit) * x[cit.col()];
                else
                    diag = *cit;
            }
            x[rit.row()] /= diag;
        }
    }
}

int main()
{
    // [[2,1,0],[1,3,2],[0,1,1]]
    NumMatrixClass<double> A(3,3,0);
    A(0,0) = 2;
    A(0,1) = 1;
    A(0,2) = 0;
    A(1,0) = 1;
    A(1,1) = 3;
    A(1,2) = 2;
    A(2,0) = 0;
    A(2,1) = 1;
    A(2,2) = -4;

    // [1,2,3]
    std::vector<float> b(3,0);
    b[0] = 1;
    b[1] = 2;
    b[2] = 3;

    std::vector<float> x(3,1);
    gaussseidel(A,b,x, 100);

    // [0,1,-0.5]
    for (unsigned int i=0; i<x.size(); i++)
    {
        std::cout << x[i] << "\n";
    }

    return 0;
}
